/*
*		Framework for Interactive GameWall Application
*				   ServerApp Framework
*
*	Written by Aleksander Aanesland Elvemo and Vegard Gamnes
*					  Spring 2013
*/

//The following function will work as a prototype for all server applications using FIGA
var GameWallServer = function () {

	//Importing required libraries for web hosting and communication with UserApps/GameWallApps
	var express = require('express')
  	, app = express()
  	, server = require('http').createServer(app)
  	, io = require('socket.io').listen(server, { log: false });	

  	//The port the ServerApp will listen to
	this.port = 8080;
	//If true, the ServerApp will store disconnected connections for reconnections
	this.storeOldConnections = true;
	//This object contains all UserApp connections, GameWallApp connections and old connections
	this.connections = {
		userApps: [],
		gamewallApps: [],
		oldConnections: {}
	};

	//This array contains all function bindings to a connection, that triggers if the respective key is received
	this.socketBindings = [];
	
	//The init function starts up the ServerApp, and makes it listen to the defined port for incoming connections
	this.init = function (directory){

		server.listen(this.port);

		//The following command setup the web hosting directory
		app.configure(function() {
		  app.use(express.static(directory + '/'));
		});
		//The following command makes the ServerApp send the UserApp.html when the root dir is requested (index file)
		app.get('/', function (req, res) {
		  res.sendfile(directory + '/UserApp.html');
		});
		//The following command makes the FigaUser.js file available from the framework directory
		app.get('/FIGAUser.js', function (req, res) {
			//console.log('Getting file ');
		  	res.sendfile(__dirname + '/FIGAUser.js');
		});
		//The following command makes the FigaGameWall.js file available from the framework directory
		app.get('/FIGAGameWall.js', function (req, res) {
			//console.log('Getting file ');
		  	res.sendfile(__dirname + '/FIGAGameWall.js');
		});

		//The following function is exectuted when a new connection is requested
		io.sockets.on('connection', function (socket) {
			
			//Check if the ServerApp stores old connections, and check if the new one matches an old one
			if(this.storeOldConnections){
				var thisIP = socket.manager.handshaken[socket.id].address.address;
				if(this.connections.oldConnections[thisIP]){
					//There is a match! Transfer all state information from the old connection to the new one
					this.reSyncSocket(socket, this.connections.oldConnections[thisIP]);
					delete this.connections.oldConnections[thisIP];
				}
			}

			//After connection, a UserApp will identify itself as a UserApp with this event. Bind all corresponding functions to the UserApp.
			socket.on('verifyUserApp', function(){
				console.log('UserApp connected');
				this.bindSocket(socket, 'userApp');
				socket.join('userApps');
				this.addUserApp(socket);
			}.bind(this));
			//After connection, a GameWallApp will identify itself as a GameWallApp with this event. Bind all corresponding functions to the GameWallApp.
			socket.on('verifyGameWallApp', function(){
				console.log('GameWallApp connected');
				this.bindSocket(socket, 'gamewallApp');
				socket.join('gamewallApps');
				this.addGameWallApp(socket);
			}.bind(this));
			//When a UserApp or GameWallApp disconnects from the ServerApp, this function is triggered
			socket.on("disconnect", function() {
				//If the ServerApp stores old connection, do it now
			  	this.addOldConnection(socket);
			  	var isUserApp = this.connections.userApps.indexOf(socket) > -1;
			  	if(isUserApp){
			  		this.removeUserApp(socket);
			  		socket.leave("userApps");
			  	}else{
			  		this.removeGameWallApp(socket);
			  		socket.leave("gamewallApps");
			  	}
			  	
			}.bind(this));

			//Finally, bind all general functions that both UserApps and GameWallApps use
			this.bindSocket(socket, 'all');

		}.bind(this));
	}

	//setPort() sets the port that the ServerApps listens to. Must be called before init()
	this.setPort = function(newPort){
		this.port = newPort;
	}
	//Adds a binding between a function and an event. Applies for both UserApps and GameWallApps
	this.addBinding = function(event, func){
		this.socketBindings.push({ event: event, channel:'all', func: func });
	}
	//Adds a binding between a function and an event. Applies for a specified channel only
	this.addChannelBinding = function(event, channel, func){
		this.socketBindings.push({ event: event, channel: channel, func: func });
	}
	//This function iterates over all known bindings, binding the connection to a function if a match is found. All UserApps are bound to the 'userApps' channel, for instance.
	this.bindSocket = function(socket, channel){
		for(var i = 0; i < this.socketBindings.length; i++){
			var binding = this.socketBindings[i];
			if(channel === binding.channel){
				socket.on(binding.event, binding.func);
			}
		}
	}
	//The ServerApp sends a message with a payload to all connections in a specific channel
	this.sendDataToChannel = function(event, data, channel){
		io.sockets.in(channel).emit(event, data);
	}
	//The ServerApp sends a message without a payload to a specific connection
	this.sendEventToSocket = function(socket, event){
		socket.emit(event);
	}
	//The ServerApp sends a message with a payload to a specific connection
	this.sendDataToSocket = function(socket, event, data){
		socket.emit(event, data);
	}
	//Transfers the old state data from a previous disconnected connection to a new one
	this.reSyncSocket = function(socket, oldSocketInfo){
		socket.info = oldSocketInfo.info;
	}
	//Adds a UserApp connection to the userApps array
	this.addUserApp = function(userApp){
		this.connections.userApps.push(userApp);
	}
	//Removes a UserApp connection from the userApps array
	this.removeUserApp = function(userApp){
		for(var i = 0; i < this.connections.userApps.length; i++){
			var userAppInArray = this.connections.userApps[i];
			if(userApp.id == userAppInArray.id){
				this.connections.userApps.splice(i, 1);
				break;
			}
		}
	}
	//Adds a GameWallApp connection to the gamewallApps array
	this.addGameWallApp = function(gameWallApp){
		this.connections.gamewallApps.push(gameWallApp);
	}
	//Removes a GameWallApp connection from the gamewallApps array
	this.removeGameWallApp = function(gameWallApp){
		for(var i = 0; i < this.connections.gamewallApps.length; i++){
			var gameWallAppInArray = this.connections.gamewallApps[i];
			if(gameWallApp.id == gameWallAppInArray.id){
				this.connections.gamewallApps.splice(i, 1);
				break;
			}
		}
	}
	//Adds a disconnecting connection to the oldConnections hashmap, using the ip as key
	this.addOldConnection = function(socket){
		if(!this.dataObjectIsEmpty(socket.info)){
			var ip = socket.manager.handshaken[socket.id].address.address;
			var isUserApp = this.connections.userApps.indexOf(socket) > -1;
		  	if(isUserApp){
		  		this.removeUserApp(socket);
		  	}else{
		  		this.removeGameWallApp(socket);
		  	}
			this.connections.oldConnections['' + ip] = { info: socket.info, type: (isUserApp ? 'userApp' : 'gamewallApp') };
		}
	}
 	//Returns all connected UserApp connections
	this.getUserAppsConnected = function(){
		return this.connections.userApps;
	}
	//Returns all connected GameWallApp connections
	this.getGameWallAppsConnected = function(){
		return this.connections.gamewallApps;
	}
	//Returns true if the data object is empty. Used for checking if a connection contains state information necessary to store
	this.dataObjectIsEmpty = function(obj) {
	    for(var prop in obj) {
	        if(obj.hasOwnProperty(prop)){
	        	return false;
	        }
	    }
	    return true;
	}
}

//Makes it possible to import the GameWallServer-object from an external file
exports.GameWallServer = GameWallServer;